@@ -1,9 +1,10 @@
-// Copyright (c) 2006  Tel-Aviv University (Israel).
+// Copyright (c) 2005,2006,2007,2008,2009,2010,2011 Tel-Aviv University (Israel).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Arrangement_on_surface_2/include/CGAL/Arr_dcel_base.h $
-// $Id: Arr_dcel_base.h 60453 2010-12-21 17:52:03Z lrineau $
+// $URL$
+// $Id$
 // 
 //
 // Author(s)     : Ron Wein <wein@post.tau.ac.il>
@@ -35,14 +36,14 @@
 #include <CGAL/function_objects.h>
 #include <CGAL/Iterator_project.h>
 #include <CGAL/Arrangement_2/Arrangement_2_iterators.h>
-#include <boost/static_assert.hpp>
+#include <CGAL/assertions.h>
 
 
 namespace CGAL {
 
 inline void* _clean_pointer (const void* p)
 {
-  BOOST_STATIC_ASSERT(sizeof(void*) == sizeof(size_t));
+  CGAL_static_assertion(sizeof(void*) == sizeof(size_t));
   const size_t  mask = ~1;
   const size_t  val = (reinterpret_cast<size_t>(p) & mask);
 
@@ -527,12 +528,13 @@ public:
       this->p_v = v;
   }
 
-  /*! Check whether the halfedge lies on the boundary of an inner CCB. */
-  bool is_on_inner_ccb () const
+  /*! Check whether the halfedge lies on the boundary of an outer CCB. */
+  bool is_on_outer_ccb () const
   {
-    return (_is_lsb_set (this->p_comp));
+    return (!_is_lsb_set (this->p_comp));
   }
 
+
   /*!
    * Get an incident outer CCB (const version).
    * \pre The edge does not lie on an inner CCB.
@@ -560,6 +562,12 @@ public:
     this->p_comp = oc;
   }
 
+  /*! Check whether the halfedge lies on the boundary of an inner CCB. */
+  bool is_on_inner_ccb () const
+  {
+    return (_is_lsb_set (this->p_comp));
+  }
+
   /*!
    * Get an incident inner CCB (const version).
    * \pre The edge lies on an inner CCB.
