@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org); you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; version 2.1 of the License.
-// See the file LICENSE.LGPL distributed with CGAL.
+// published by the Free Software Foundation; either version 3 of the License,
+// or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Kinetic_data_structures/include/CGAL/Kinetic/internal/debug_counters.h $
-// $Id: debug_counters.h 56668 2010-06-09 08:45:58Z sloriot $
+// $URL$
+// $Id$
 // 
 //
 // Author(s)     : Daniel Russel <drussel@alumni.princeton.edu>
@@ -22,11 +22,11 @@
 
 namespace CGAL { namespace Kinetic {
 namespace internal {
-  extern unsigned int zero_certificates__;
-  extern unsigned int function_degeneracies__;
-  extern unsigned int io_errors__;
-  extern unsigned int audit_failures__;
+  CGAL_EXPORT extern unsigned int zero_certificates__;
+  CGAL_EXPORT extern unsigned int function_degeneracies__;
+  CGAL_EXPORT extern unsigned int io_errors__;
+  CGAL_EXPORT extern unsigned int audit_failures__;
 
-  void write_debug_counters(std::ostream &out);
+  CGAL_EXPORT void write_debug_counters(std::ostream &out);
 }
 } } //namespace CGAL::Kinetic
