@@ -1,9 +1,10 @@
 // Copyright (c) 1999  Tel-Aviv University (Israel).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Largest_empty_rect_2/include/CGAL/Largest_empty_iso_rectangle_2.h $
-// $Id: Largest_empty_iso_rectangle_2.h 56667 2010-06-09 07:37:13Z sloriot $
+// $URL$
+// $Id$
 // 
 //
 // Author(s)     : Eli Packer (algorithm), Andreas Fabri (cgal conformance)
@@ -312,8 +313,12 @@ public:
     {}
 
     ~Point_data() {
-      delete right_tent;
-      delete left_tent;
+      if(right_tent != NULL){
+        delete right_tent;
+      }
+      if (left_tent != NULL){
+        delete left_tent;
+      }
     }
   }; 
 
@@ -548,21 +553,21 @@ private:
       --iter;
   }
 
-  void determine_first_two_iters(typename Point_data_set_of_y::iterator &iter1,
-				 typename Point_data_set_of_y::iterator &iter2,
-				 typename Point_data_set_of_y::iterator &iter3,
-				 bool &first_iter_is_right,
-				 bool &second_iter_is_right,
-				 bool &third_iter_is_right)
+  void determine_first_two_iters(typename Point_data_set_of_y::iterator& iter1,
+				 typename Point_data_set_of_y::iterator& iter2,
+				 typename Point_data_set_of_y::iterator& iter3,
+				 bool& first_iter_is_right,
+				 bool& second_iter_is_right,
+				 bool& third_iter_is_right)
   {
-    if(first_iter_is_right) {
-      if(second_iter_is_right) {
+    if (first_iter_is_right) {
+      if (second_iter_is_right) {
         iter1 = iter2;
         iter2 = iter3;
         first_iter_is_right = second_iter_is_right;
         second_iter_is_right = third_iter_is_right;
       } else {
-        if(third_iter_is_right) {
+        if (third_iter_is_right) {
           iter1 = iter2;
           iter2 = iter3;
           first_iter_is_right = second_iter_is_right;
@@ -573,8 +578,8 @@ private:
         }
       }
     } else {
-      if(second_iter_is_right) {
-        if(third_iter_is_right) {
+      if (second_iter_is_right) {
+        if (third_iter_is_right) {
           iter2 = iter3;
           second_iter_is_right = third_iter_is_right;  
         } else {
@@ -816,11 +821,10 @@ Largest_empty_iso_rectangle_2<T>::remove(const Point& _p)
   if(iter1 == x_sorted.end() || (*iter1)->type != REG)
     return(false);
 
-  delete((*iter1)->right_tent);
-  delete((*iter2)->left_tent);
-
+  Point_data* ptr  = *iter1;
   x_sorted.erase(iter1);
   y_sorted.erase(iter2);
+  delete ptr;
 
   return(true);
 }
@@ -989,7 +993,7 @@ Largest_empty_iso_rectangle_2<T>::phase_2_on_bot()
     iter2,iter3,first_iter,
     beyond = Point_data_list.end();
   int points_removed = 0, 
-      size = Point_data_list.size();
+    size = static_cast<int>(Point_data_list.size());
 
   get_next_for_bot(iter1,beyond);
   first_iter = iter1;
@@ -1038,7 +1042,7 @@ Largest_empty_iso_rectangle_2<T>::phase_2_on_top()
     iter2, iter3, first_iter,
     beyond = Point_data_list.end();
   int points_removed = 0,
-    size = Point_data_list.size();
+    size = static_cast<int>(Point_data_list.size());
 
   get_next_for_top(iter1,beyond);
   iter2 = iter1;
@@ -1084,7 +1088,7 @@ Largest_empty_iso_rectangle_2<T>::phase_2_on_left()
     iter2, iter3, first_iter,
     beyond = Point_data_list.end();
   int points_removed = 0,
-      size = Point_data_list.size();
+    size = static_cast<int>(Point_data_list.size());
 
   get_next_for_left(iter1,beyond);
   first_iter = iter1;
@@ -1129,7 +1133,7 @@ Largest_empty_iso_rectangle_2<T>::phase_2_on_right()
   typename std::list<Point_data *>::iterator iter1 = Point_data_list.begin(),
     iter2, iter3, first_iter, 
     beyond = Point_data_list.end();
-  int points_removed = 0,size = Point_data_list.size();
+  int points_removed = 0,size = static_cast<int>(Point_data_list.size());
 
   get_next_for_right(iter1,beyond);
   first_iter = iter1;
@@ -1180,8 +1184,14 @@ template<class T>
 void 
 Largest_empty_iso_rectangle_2<T>::phase_3()
 {
-  bool first_iter_is_right = true, second_iter_is_right = true; // init for compiler warning
-  bool third_iter_is_right, first_exist,second_exist,third_exist;
+  // init for compiler warning
+  bool first_iter_is_right(true);
+  bool second_iter_is_right(true);
+  bool third_iter_is_right(true);
+  bool first_exist(true);
+  bool second_exist(true);
+  bool third_exist(true);
+  
   typename Point_data_set_of_y::iterator iter, last_iter = y_sorted.end();
   typename Point_data_set_of_y::iterator iter1, iter2, iter3, 
                                 right_iter, left_iter, last = last_iter;
