@@ -1,9 +1,10 @@
 // Copyright (c) 1997-2001  ETH Zurich (Switzerland).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Min_annulus_d/include/CGAL/Min_annulus_d.h $
-// $Id: Min_annulus_d.h 56667 2010-06-09 07:37:13Z sloriot $
+// $URL$
+// $Id$
 // 
 //
 // Author(s)     : Sven Schoenherr <sven@inf.ethz.ch>
@@ -29,7 +30,7 @@
 #include <CGAL/QP_solver/functors.h>
 #include <CGAL/QP_solver/QP_full_filtered_pricing.h>
 #include <CGAL/QP_solver/QP_full_exact_pricing.h>
-#include <boost/iterator/counting_iterator.hpp>
+#include <CGAL/boost/iterator/counting_iterator.hpp>
 #include <boost/iterator/transform_iterator.hpp>
 
 // here is how it works. We have d+2 variables: 
@@ -310,7 +311,7 @@ public:
   // access to point set
   int  ambient_dimension( ) const { return d; }
     
-  int  number_of_points( ) const { return points.size(); }
+  int  number_of_points( ) const { return static_cast<int>(points.size()); }
     
   Point_iterator  points_begin( ) const { return points.begin(); }
   Point_iterator  points_end  ( ) const { return points.end  (); }
@@ -344,8 +345,8 @@ public:
 						   std::bind2nd( std::divides<int>(), 2)));
   }
     
-  int  number_of_inner_support_points() const { return inner_indices.size();}
-  int  number_of_outer_support_points() const { return outer_indices.size();}
+  int  number_of_inner_support_points() const { return static_cast<int>(inner_indices.size());}
+  int  number_of_outer_support_points() const { return static_cast<int>(outer_indices.size());}
     
   Inner_support_point_iterator
   inner_support_points_begin() const
@@ -485,7 +486,7 @@ public:
   template < class InputIterator >
   void
   insert( InputIterator first, InputIterator last)
-  { CGAL_optimisation_precondition_code( int old_n = points.size());
+  { CGAL_optimisation_precondition_code( std::size_t old_n = points.size());
   points.insert( points.end(), first, last);
   set_dimension();
   CGAL_optimisation_precondition_msg( check_dimension( old_n),
@@ -586,7 +587,7 @@ private:
     
   // check dimension of input points
   bool
-  check_dimension( unsigned int  offset = 0)
+  check_dimension( std::size_t  offset = 0)
   { return ( std::find_if( points.begin()+offset, points.end(),
 			   CGAL::compose1_1( std::bind2nd(
 							  std::not_equal_to<int>(), d),
@@ -636,7 +637,7 @@ private:
       c_vector[ 2*j+1] = -sum;
     }
         
-    LP lp (2*points.size(), d+2, 
+    LP lp (2*static_cast<int>(points.size()), d+2, 
 	   A_iterator ( boost::counting_iterator<int>(0), 
 			A_matrix (d, da_coord, points.begin())),
 	   B_iterator ( boost::counting_iterator<int>(0), 
